@@ -1,5 +1,6 @@
 /*
  * Copyright © 2016 Canonical Ltd.
+ * Copyright © 2022 Guido Berhoerster <guido+gitlab.com@berhoerster.name>
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3, as published
@@ -172,6 +173,67 @@ TEST_F(ApplicationInfoDesktop, KeyfileErrors)
                  std::runtime_error);
 }
 
+TEST_F(ApplicationInfoDesktop, UbuntuFallback)
+{
+    lomiri::app_launch::Application::Info::Orientations fallbackOrientations =
+        {portrait : true, landscape : false, invertedPortrait : false, invertedLandscape : false};
+    lomiri::app_launch::Application::Info::Orientations lomiriOrientations =
+        {portrait : false, landscape : true, invertedPortrait : false, invertedLandscape : false};
+    auto keyfile = defaultKeyfile();
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Ubuntu-Default-Department-ID", "foo");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Ubuntu-Splash-Title", "baz");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Ubuntu-Splash-Image", "/quux.png");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Ubuntu-Splash-Color", "red");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Ubuntu-Splash-Color-Header", "green");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Ubuntu-Splash-Color-Footer", "blue");
+    g_key_file_set_boolean(keyfile.get(), DESKTOP, "X-Ubuntu-Splash-Show-Header", true);
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Ubuntu-Supported-Orientations", "portrait;");
+    g_key_file_set_boolean(keyfile.get(), DESKTOP, "X-Ubuntu-Rotates-Window-Contents", true);
+    g_key_file_set_boolean(keyfile.get(), DESKTOP, "X-Ubuntu-Touch", true);
+    g_key_file_set_boolean(keyfile.get(), DESKTOP, "X-Ubuntu-Single-Instance", true);
+
+    auto fallbackAppinfo = lomiri::app_launch::app_info::Desktop(simpleAppID(), keyfile, "/", {},
+                                                                  lomiri::app_launch::app_info::DesktopFlags::NONE, nullptr);
+    EXPECT_EQ("foo", fallbackAppinfo.defaultDepartment().value());
+    EXPECT_EQ("baz", fallbackAppinfo.splash().title.value());
+    EXPECT_EQ("/quux.png", fallbackAppinfo.splash().image.value());
+    EXPECT_EQ("red", fallbackAppinfo.splash().backgroundColor.value());
+    EXPECT_EQ("green", fallbackAppinfo.splash().headerColor.value());
+    EXPECT_EQ("blue", fallbackAppinfo.splash().footerColor.value());
+    EXPECT_TRUE(fallbackAppinfo.splash().showHeader.value());
+    EXPECT_EQ(fallbackOrientations, fallbackAppinfo.supportedOrientations());
+    EXPECT_TRUE(fallbackAppinfo.rotatesWindowContents().value());
+    EXPECT_TRUE(fallbackAppinfo.supportsLomiriLifecycle().value());
+    EXPECT_TRUE(fallbackAppinfo.singleInstance().value());
+
+    // override X-Ubuntu*
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Lomiri-Default-Department-ID", "bar");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Lomiri-Splash-Title", "bla");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Lomiri-Splash-Image", "/waldo.png");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Lomiri-Splash-Color", "yellow");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Lomiri-Splash-Color-Header", "cyan");
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Lomiri-Splash-Color-Footer", "magenta");
+    g_key_file_set_boolean(keyfile.get(), DESKTOP, "X-Lomiri-Splash-Show-Header", false);
+    g_key_file_set_string(keyfile.get(), DESKTOP, "X-Lomiri-Supported-Orientations", "landscape;");
+    g_key_file_set_boolean(keyfile.get(), DESKTOP, "X-Lomiri-Rotates-Window-Contents", false);
+    g_key_file_set_boolean(keyfile.get(), DESKTOP, "X-Lomiri-Touch", false);
+    g_key_file_set_boolean(keyfile.get(), DESKTOP, "X-Lomiri-Single-Instance", false);
+
+    auto lomiriAppinfo = lomiri::app_launch::app_info::Desktop(simpleAppID(), keyfile, "/", {},
+                                                                lomiri::app_launch::app_info::DesktopFlags::NONE, nullptr);
+    EXPECT_EQ("bar", lomiriAppinfo.defaultDepartment().value());
+    EXPECT_EQ("bla", lomiriAppinfo.splash().title.value());
+    EXPECT_EQ("/waldo.png", lomiriAppinfo.splash().image.value());
+    EXPECT_EQ("yellow", lomiriAppinfo.splash().backgroundColor.value());
+    EXPECT_EQ("cyan", lomiriAppinfo.splash().headerColor.value());
+    EXPECT_EQ("magenta", lomiriAppinfo.splash().footerColor.value());
+    EXPECT_FALSE(lomiriAppinfo.splash().showHeader.value());
+    EXPECT_EQ(lomiriOrientations, lomiriAppinfo.supportedOrientations());
+    EXPECT_FALSE(lomiriAppinfo.rotatesWindowContents().value());
+    EXPECT_FALSE(lomiriAppinfo.supportsLomiriLifecycle().value());
+    EXPECT_FALSE(lomiriAppinfo.singleInstance().value());
+}
+
 TEST_F(ApplicationInfoDesktop, KeyfileIconPatterns)
 {
     auto defkeyfile = defaultKeyfile();
